@@ -3,6 +3,7 @@ use std::io;
 use std::io::prelude::*;
 use std::path::{Path, PathBuf};
 
+use clap::arg_enum;
 use serde::{Deserialize, Serialize};
 
 use crate::error::FatalError;
@@ -56,6 +57,10 @@ pub trait ConfigSource {
         None
     }
 
+    fn consolidate_pushes(&self) -> Option<bool> {
+        None
+    }
+
     fn pre_release_commit_message(&self) -> Option<&str> {
         None
     }
@@ -126,6 +131,7 @@ pub struct Config {
     pub dev_version_ext: Option<String>,
     pub no_dev_version: Option<bool>,
     pub consolidate_commits: Option<bool>,
+    pub consolidate_pushes: Option<bool>,
     pub pre_release_commit_message: Option<String>,
     // depreacted
     pub pro_release_commit_message: Option<String>,
@@ -180,6 +186,9 @@ impl Config {
         if let Some(consolidate_commits) = source.consolidate_commits() {
             self.consolidate_commits = Some(consolidate_commits);
         }
+        if let Some(consolidate_pushes) = source.consolidate_pushes() {
+            self.consolidate_pushes = Some(consolidate_pushes);
+        }
         if let Some(pre_release_commit_message) = source.pre_release_commit_message() {
             self.pre_release_commit_message = Some(pre_release_commit_message.to_owned());
         }
@@ -238,14 +247,11 @@ impl Config {
     }
 
     pub fn push_remote(&self) -> &str {
-        self.push_remote
-            .as_ref()
-            .map(|s| s.as_str())
-            .unwrap_or("origin")
+        self.push_remote.as_deref().unwrap_or("origin")
     }
 
     pub fn registry(&self) -> Option<&str> {
-        self.registry.as_ref().map(|s| s.as_str())
+        self.registry.as_deref()
     }
 
     pub fn disable_release(&self) -> bool {
@@ -268,10 +274,7 @@ impl Config {
     }
 
     pub fn dev_version_ext(&self) -> &str {
-        self.dev_version_ext
-            .as_ref()
-            .map(|s| s.as_str())
-            .unwrap_or("alpha.0")
+        self.dev_version_ext.as_deref().unwrap_or("alpha.0")
     }
 
     pub fn no_dev_version(&self) -> bool {
@@ -282,17 +285,19 @@ impl Config {
         self.consolidate_commits.unwrap_or(false)
     }
 
+    pub fn consolidate_pushes(&self) -> bool {
+        self.consolidate_pushes.unwrap_or(false)
+    }
+
     pub fn pre_release_commit_message(&self) -> &str {
         self.pre_release_commit_message
-            .as_ref()
-            .map(|s| s.as_str())
+            .as_deref()
             .unwrap_or("(cargo-release) version {{version}}")
     }
 
     pub fn post_release_commit_message(&self) -> &str {
         self.post_release_commit_message
-            .as_ref()
-            .map(|s| s.as_str())
+            .as_deref()
             .unwrap_or("(cargo-release) start next development iteration {{next_version}}")
     }
 
@@ -316,24 +321,19 @@ impl Config {
 
     pub fn tag_message(&self) -> &str {
         self.tag_message
-            .as_ref()
-            .map(|s| s.as_str())
+            .as_deref()
             .unwrap_or("(cargo-release) {{crate_name}} version {{version}}")
     }
 
     pub fn tag_prefix(&self, is_root: bool) -> &str {
         // crate_name as default tag prefix for multi-crate project
         self.tag_prefix
-            .as_ref()
-            .map(|s| s.as_str())
+            .as_deref()
             .unwrap_or_else(|| if !is_root { "{{crate_name}}-" } else { "" })
     }
 
     pub fn tag_name(&self) -> &str {
-        self.tag_name
-            .as_ref()
-            .map(|s| s.as_str())
-            .unwrap_or("{{prefix}}v{{version}}")
+        self.tag_name.as_deref().unwrap_or("{{prefix}}v{{version}}")
     }
 
     pub fn disable_tag(&self) -> bool {
@@ -370,11 +370,11 @@ impl ConfigSource for Config {
     }
 
     fn push_remote(&self) -> Option<&str> {
-        self.push_remote.as_ref().map(|s| s.as_str())
+        self.push_remote.as_deref()
     }
 
     fn registry(&self) -> Option<&str> {
-        self.registry.as_ref().map(|s| s.as_str())
+        self.registry.as_deref()
     }
 
     fn disable_release(&self) -> Option<bool> {
@@ -394,7 +394,7 @@ impl ConfigSource for Config {
     }
 
     fn dev_version_ext(&self) -> Option<&str> {
-        self.dev_version_ext.as_ref().map(|s| s.as_str())
+        self.dev_version_ext.as_deref()
     }
 
     fn no_dev_version(&self) -> Option<bool> {
@@ -405,19 +405,21 @@ impl ConfigSource for Config {
         self.consolidate_commits
     }
 
+    fn consolidate_pushes(&self) -> Option<bool> {
+        self.consolidate_pushes
+    }
+
     fn pre_release_commit_message(&self) -> Option<&str> {
-        self.pre_release_commit_message.as_ref().map(|s| s.as_str())
+        self.pre_release_commit_message.as_deref()
     }
 
     // deprecated
     fn pro_release_commit_message(&self) -> Option<&str> {
-        self.pro_release_commit_message.as_ref().map(|s| s.as_str())
+        self.pro_release_commit_message.as_deref()
     }
 
     fn post_release_commit_message(&self) -> Option<&str> {
-        self.post_release_commit_message
-            .as_ref()
-            .map(|s| s.as_str())
+        self.post_release_commit_message.as_deref()
     }
 
     fn pre_release_replacements(&self) -> Option<&[Replace]> {
@@ -433,15 +435,15 @@ impl ConfigSource for Config {
     }
 
     fn tag_message(&self) -> Option<&str> {
-        self.tag_message.as_ref().map(|s| s.as_str())
+        self.tag_message.as_deref()
     }
 
     fn tag_prefix(&self) -> Option<&str> {
-        self.tag_prefix.as_ref().map(|s| s.as_str())
+        self.tag_prefix.as_deref()
     }
 
     fn tag_name(&self) -> Option<&str> {
-        self.tag_name.as_ref().map(|s| s.as_str())
+        self.tag_name.as_deref()
     }
 
     fn disable_tag(&self) -> Option<bool> {
